 -- RingSlingerPlus
 -- 5-RSP_Instagib.lua
 -- 12.07.2023
 -- Sirexer

local railboost_powervert = (railboost_power.value / 7)

//InstaShield
local function InstaShield_rsp(player, mo)
	if (player.instacalmdown == 0
	and player.insta_count == 100)
		if player.ins_shield == false
			player.ins_shield = true
			player.instacalmdown = $ + 10
			S_StartSound(nil, sfx_insshi, player)
			return true
		end
	end
end

//The player will have their InstaShield depleted when they spawn.
addHook("PlayerSpawn", function(player)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		player.ins_shield = false
		player.instacalmdown = 0
		player.insta_count = 0
	end
end)

//Works only on InstaGib gametypes.
addHook("PlayerThink", function(player)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		if player.mo
		and player.mo.valid
		and player.playerstate == PST_LIVE
			//Giving the RailRing.
			if player.ringweapons != 32
				player.ringweapons = 32
			end
			//And infinite ammo.
			if player.powers[pw_railring] != 1
				player.powers[pw_railring] = 1
			end
			//The player has only a RailRing in his hands.
			if player.currentweapon != 6
				player.currentweapon = 6
			end
			//Do not show surcharge for 0 rings.
			player.ammoremovaltimer = 0
			player.ammoremoval = 0
			player.ammoremovalweapon = 0
			//The player needs to press the custom button 1 to turn on the InstaShield.
			if cv_instashield.value == 1
				if (player.cmd.buttons & RSP.buttons[player.rsp_instashield_button].button)
					InstaShield_rsp(player)
				end
				//What will happen to the player.
				if player.ins_shield == true
					player.insta_count = $ - 5 / 2
					player.mo.colorized = true
					player.powers[pw_flashing] = 1
				end
				//Turn off the InstaShield if the player has 0 rings.
				if (player.ins_shield == true
				and player.insta_count == 0)
					player.mo.colorized = false
					player.ins_shield = false
					player.instacalmdown = 10
				end
				//Charging InstaShield.
				if (player.ins_shield == false
				and player.insta_count != 100
				and player.instacalmdown == 0)
					player.instacalmdown = 2
					player.insta_count = $ + 1
				end
				//Calmdown.
				if player.instacalmdown != 0
				and player.instacalmdown != nil
					player.instacalmdown = $ - 1
				end
			end
		end
	end
end, MT_PLAYER)

//Killing instantly if he received damage from the player.
addHook("MobjDamage", function(target, inflictor, source, name)
	if not target.player then
		return false
	end
	if inflictor then
		source = inflictor.target
	end
	if (cv_instagib.value == 1 and not G_IsNoInstaGametype()) or (cv_rspsuddendeath.value == 1 and gametype != GT_HOTPOTATO)
	and source and source.player then
		target.player.rings = 0
		target.player.powers[pw_shield] = SH_NONE
		P_KillMobj(target, inflictor, source)
	end
end, MT_PLAYER)

//Killing Rail Ring
addHook("MobjThinker", function(ring)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		if ring
		and ring.valid
		and ring.type == MT_REDRING
		and ring.health
			if cv_rbenabled.value == 1
				if (ring.flags2 & MF2_RAILRING)
					if ring.state == ring.info.deathstate
					or ring.state == S_SPRK2
					or ring.state == S_SPRK3
					P_KillMobj(ring)
					end
				end
			end
		end
	end
end, MT_REDRING)

//RailBoost
addHook("MobjDeath", function(ring)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		if ring
		and ring.valid
		and ring.type == MT_REDRING
			if cv_rbenabled.value == 1
				if (ring.flags2 & MF2_RAILRING)
					for player in players.iterate
						if player.mo
						and player.mo.valid
						and player.playerstate == PST_LIVE
						local dist = R_PointToDist2(ring.x, ring.y, player.mo.x, player.mo.y)
						local dist2 = P_AproxDistance(0, player.mo.z-ring.z)
						local angle = R_PointToAngle2(ring.x, ring.y, player.mo.x, player.mo.y)
							if dist < 128*FRACUNIT
							and dist2 < 64*FRACUNIT
							local horizontalAngle = R_PointToAngle2(ring.x, ring.y, player.mo.x, player.mo.y)
							local verticalAngle = R_PointToAngle2(FixedHypot(ring.x, ring.y), ring.z+(ring.height/2),FixedHypot(player.mo.x, player.mo.y), player.mo.z+(player.mo.height/2))
							P_SetObjectMomZ(player.mo, railboost_powervert*sin(verticalAngle), true)
							P_Thrust(player.mo, horizontalAngle, railboost_power.value*abs(cos(verticalAngle)))
							end
						end
					end
				end
			end
		end
	end
end, MT_REDRING)

//Removing the red ring
addHook("MobjThinker", function(mo)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		if not (mo.flags2 & MF2_RAILRING)
			P_RemoveMobj(mo)
		end
	end
end, MT_REDRING)

//Making objects invisible.
local function Insta_RemObjects(mo)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		mo.state = S_DISABLEOBJECT
    end
end

addHook("MobjThinker", Insta_RemObjects, MT_LHRT)
addHook("MobjThinker", Insta_RemObjects, MT_CORK)
addHook("MobjThinker", Insta_RemObjects, MT_SPINFIRE)
addHook("MobjThinker", Insta_RemObjects, MT_INFINITYRING)
addHook("MobjThinker", Insta_RemObjects, MT_AUTOMATICRING)
addHook("MobjThinker", Insta_RemObjects, MT_BOUNCERING)
addHook("MobjThinker", Insta_RemObjects, MT_SCATTERRING)
addHook("MobjThinker", Insta_RemObjects, MT_GRENADERING)
addHook("MobjThinker", Insta_RemObjects, MT_EXPLOSIONRING)
addHook("MobjThinker", Insta_RemObjects, MT_RAILRING)
addHook("MobjThinker", Insta_RemObjects, MT_AUTOPICKUP)
addHook("MobjThinker", Insta_RemObjects, MT_BOUNCEPICKUP)
addHook("MobjThinker", Insta_RemObjects, MT_SCATTERPICKUP)
addHook("MobjThinker", Insta_RemObjects, MT_GRENADEPICKUP)
addHook("MobjThinker", Insta_RemObjects, MT_EXPLODEPICKUP)
addHook("MobjThinker", Insta_RemObjects, MT_RAILPICKUP)

//Deleting Spin Fire.
addHook("MobjThinker", function(mo)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		mo.state = S_NULL
    end
end, MT_SPINFIRE)

//The player cannot take these things.
local function Insta_DontTouch(mo, toucher)
	if cv_instagib.value == 1 and not G_IsNoInstaGametype()
		return true
	end
end

addHook("TouchSpecial", Insta_DontTouch, MT_EXPLOSIONRING)
addHook("TouchSpecial", Insta_DontTouch, MT_AUTOMATICRING)
addHook("TouchSpecial", Insta_DontTouch, MT_SCATTERRING)
addHook("TouchSpecial", Insta_DontTouch, MT_BOUNCERING)
addHook("TouchSpecial", Insta_DontTouch, MT_RAILRING)
addHook("TouchSpecial", Insta_DontTouch, MT_GRENADERING)
addHook("TouchSpecial", Insta_DontTouch, MT_INFINITYRING)
addHook("TouchSpecial", Insta_DontTouch, MT_EXPLODEPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_AUTOPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_SCATTERPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_BOUNCEPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_RAILPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_GRENADEPICKUP)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD1)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD2)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD3)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD4)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD5)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD6)
addHook("TouchSpecial", Insta_DontTouch, MT_EMERALD7)
